/* --COPYRIGHT--,BSD
* Copyright (c) 2014, Texas Instruments Incorporated
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
* * Redistributions of source code must retain the above copyright
* notice, this list of conditions and the following disclaimer.
*
* * Redistributions in binary form must reproduce the above copyright
* notice, this list of conditions and the following disclaimer in the
* documentation and/or other materials provided with the distribution.
*
* * Neither the name of Texas Instruments Incorporated nor the names of
* its contributors may be used to endorse or promote products derived
* from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
* OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* --/COPYRIGHT--*/

#include <intrinsics.h>
#include <string.h>

#include <stdint.h>
#include "TI_MSP430.h"
#include "TI_MSP430_hardware_board.h"
#include "timer0.h"
#include "timer2.h"
#include "host_interface.h"
#include "comparator.h"

#include "USB_API/USB_Common/device.h"
#include "USB_API/USB_Common/types.h"               //Basic Type declarations
#include "USB_API/USB_Common/usb.h"                 //USB-specific functions

#include "F5xx_F6xx_Core_Lib/HAL_UCS.h"
#include "F5xx_F6xx_Core_Lib/HAL_PMM.h"

#include "USB_API/USB_CDC_API/UsbCdc.h"
#include "usbConstructs.h"
#include "USB_config/descriptors.h"

//******************************************************************************
#define MAX_STR_LENGTH 32
//******************************************************************************
void InitMCU(void);
void Init_Ports (void);
void Init_Clock (void);
void USBCommunicationTask(void);

// Globals & Externs
volatile uint8_t LMT01_XYBufferReady = 0;
extern uint8_t lmt01_measure_mode;
volatile BYTE bCDCDataReceived_event = FALSE;   //Indicates data has been received without an open rcv operation
WORD w;
uint8_t bufferX[MAX_BUF_LENGTH];
uint8_t bufferY[MAX_BUF_LENGTH];
uint8_t xy_buf_length = 2;
uint8_t save_xy_buf_length;
uint8_t *mbuf=bufferX;
uint8_t *ubuf;

uint8_t cmdResponseString[MAX_STR_LENGTH] = "";
uint8_t single_shot_state=0;
uint8_t cont_graph_state=0;
uint32_t err_count = 0;
//******************************************************************************

void main(void)
{
  
  WDTCTL = WDTPW+WDTHOLD;                                                      // Stop WDT 
  
  // Init XFER_LED GPIO: output and high 
  TI_LMT01_XFER_LED_PxOUT |= TI_LMT01_XFER_LED_PIN; 
  TI_LMT01_XFER_LED_PxSEL &= ~TI_LMT01_XFER_LED_PIN;
  TI_LMT01_XFER_LED_PxDIR |= TI_LMT01_XFER_LED_PIN;                                       

  // Init TEST_LED gpio: output and high 
  TI_LMT01_TEST_LED_PxOUT |= TI_LMT01_TEST_LED_PIN;  
  TI_LMT01_TEST_LED_PxSEL &= ~TI_LMT01_TEST_LED_PIN;
  TI_LMT01_TEST_LED_PxDIR |= TI_LMT01_TEST_LED_PIN;   

  // Init Time Measurement GPIO: input & select special function (Transistor Mode)                            
  TI_LMT01_GPIO_PULSE_COUNT_TRAN_PxDIR &= ~TI_LMT01_GPIO_PULSE_COUNT_TRAN_PIN; 
  TI_LMT01_GPIO_PULSE_COUNT_TRAN_PxSEL |= TI_LMT01_GPIO_PULSE_COUNT_TRAN_PIN;

  // Init CAP_INP GPIO: input & select special function: timer capture input (Transistor Mode)
  TI_LMT01_GPIO_CAP_INP_TRAN_PxDIR &= ~TI_LMT01_GPIO_CAP_INP_TRAN_PIN;
  TI_LMT01_GPIO_CAP_INP_TRAN_PxSEL |= TI_LMT01_GPIO_CAP_INP_TRAN_PIN;

  // Init Power GPIO: output and low
  TI_LMT01_GPIO_POWER_PxOUT &= ~TI_LMT01_GPIO_POWER_PIN;                               
  TI_LMT01_GPIO_POWER_PxDIR |= TI_LMT01_GPIO_POWER_PIN;  
  
  // oscillator fail if XFER_LED keeps flashing after InitMCU     
  InitMCU();  
  
  // initialize the comparator: testing transistor mode (disable comparatorB init)
  compB_init();
  
  // init timer0_A0 
  // Note: In TRANSISTOR Mode timer0A0 clock input is connected to transistor output
  // timer2 A1 is used to capture first pulse and timer2 A0 is 50ms stop watch timer 
  timer0_A0_init();   
 
  // init timer2_A0 
  // Note: In COMPARATOR Mode timer2A0 clock input is connected to comparator output
  // timer0 A1 is used to capture first pulse and timer0 A0 is 50ms stop watch timer
  timer2_A0_init();
  
  while (1)
  {
//    __bis_SR_register(LPM0_bits + GIE);                                        // Enter LPM0, enable interrupts
    __no_operation();                                                          // For debugger 
    USBCommunicationTask();
    __no_operation();                                                          // For debugger     
  }
}

//******************************************************************************
/**
* @brief  Local functions.                           
*/

/**
* @brief Function Name:  InitMCU.                                                
* @brief Description  :  Initializes the MSP430 peripherals and modules.
* @param parameters   :  none                                                   
* @return Value       :  none                                                   
*/   
//******************************************************************************
void InitMCU(void)
{
  
  __disable_interrupt();                                                       // Disable global interrupts 
//    Init_Ports();                                                              //Init ports (do first ports because clocks do change ports)
    SetVCore(3);
    Init_Clock();                                                              //Init clocks

#if 1
    USB_init();                 //Init USB

    USB_setEnabledEvents(
        kUSB_VbusOnEvent + kUSB_VbusOffEvent + kUSB_receiveCompletedEvent
        + kUSB_dataReceivedEvent + kUSB_UsbSuspendEvent + kUSB_UsbResumeEvent +
        kUSB_UsbResetEvent);

    //Check if we're already physically attached to USB, and if so, connect to it
    //This is the same function that gets called automatically when VBUS gets attached.
    if (USB_connectionInfo() & kUSB_vbusPresent){
        USB_handleVbusOnEvent();
    }
#endif
  __enable_interrupt();                                                        // enable global interrupts

}

//******************************************************************************
/**
* @brief Function Name: USBCommunicationTask     .                                             
* @brief Description  : Dumps data to the host over USB.
* @param parameters   : none                                                    
* @return Value       : none                                                    
*/   
//******************************************************************************
void USBCommunicationTask(void)
{
  WORD bytesSent, bytesReceived;
  uint16_t count;
  uint8_t send_error=0, receive_error=0, send_response, i;
  
        switch (USB_connectionState())
        {
            case ST_USB_DISCONNECTED:
//                __bis_SR_register(LPM3_bits + GIE);                     //Enter LPM3 w/interrupt
                _NOP();
                break;

            case ST_USB_CONNECTED_NO_ENUM:
                break;

            case ST_ENUM_ACTIVE:
                if (USBCDC_intfStatus(CDC0_INTFNUM,&bytesSent, &bytesReceived) & kUSBCDC_waitingForSend)
                    err_count++;
                // Exit LPM because of a data-receive event, and fetch the received data 
                if(bCDCDataReceived_event)
                {
                  bCDCDataReceived_event = FALSE;                                  // Clear flag early -- just in case execution breaks below because of an error
                  count = cdcReceiveDataInBuffer(cmdResponseString,MAX_STR_LENGTH,CDC0_INTFNUM);         // Count has the number of bytes received into dataBuffer    
                  send_response = handleHostCommand(cmdResponseString,count);
                  if (send_response)
                  {
                    if(cdcSendDataInBackground((BYTE*)cmdResponseString,MAX_STR_LENGTH,CDC0_INTFNUM,0))             // Send data to host
                    {
                      send_error = 1;                                            // Something went wrong -- exit
                      break;
                    }
                  }                  
                  for(i=0;i<MAX_STR_LENGTH;i++) 
                    cmdResponseString[i] = NULL;
                } else if (cont_graph_state == 1) 
                {
                  if (LMT01_XYBufferReady)
                  {
                    LMT01_XYBufferReady = 0;
                    if (cdcSendDataInBackground((BYTE*)ubuf,xy_buf_length,CDC0_INTFNUM,0))
                    {
                        bCDCDataReceived_event = FALSE;
                        USBCDC_abortSend(&w,CDC0_INTFNUM);                     //Operation probably still open; cancel it
                    }
                    if ((lmt01_measure_mode == TWO_TIMER_SYNC_TRANSISTOR_MODE) ||
                        (lmt01_measure_mode == TWO_TIMER_ASYNC_TRANSISTOR_MODE))
                      TI_LMT01_XFER_LED_PxOUT ^= TI_LMT01_XFER_LED_PIN;                // Toggle XFER_LED
                    else if ((lmt01_measure_mode == TWO_TIMER_SYNC_COMPARATOR_MODE) ||
                             (lmt01_measure_mode == TWO_TIMER_ASYNC_COMPARATOR_MODE))
                      TI_LMT01_TEST_LED_PxOUT ^= TI_LMT01_TEST_LED_PIN;                // Toggle TEST_LED                    
                  }
                }       
                break;

            case ST_ENUM_SUSPENDED:
//                __bis_SR_register(LPM3_bits + GIE);             //Enter LPM3 until a resume or VBUS-off event
                break;

            case ST_ENUM_IN_PROGRESS:
                break;

            case ST_NOENUM_SUSPENDED:
//                __bis_SR_register(LPM3_bits + GIE);
                break;

            case ST_ERROR:
                _NOP();
                break;

            default:;
        }                                                                            //end of switch-case sentence
        if(send_error || receive_error)
        { 
          //TO DO: User can place code here to handle error
        }        
}
//******************************************************************************
void Init_Clock(void)
{
  // Enable XT2 XIN/XOUT Pins
  P5SEL |= 0x0C;                            // Select XIN, XOUT on P5.3 and P5.2
  UCSCTL6 &= ~XT2OFF;                       // Enable XT2  
  UCSCTL6 |= XT2DRIVE_3; 
  UCSCTL3 |= SELREF_2;                      // FLLref = REFO
                                            // Since LFXT1 is not used,
                                            // sourcing FLL with LFXT1 can cause
                                            // XT1OFFG flag to set

  // ACLK=REFO,SMCLK=DCO,MCLK=DCO
  UCSCTL4 = SELA__REFOCLK + SELS__DCOCLKDIV + SELM__DCOCLKDIV;
  UCSCTL0 = 0x0000;                                                            // Set lowest possible DCOx, MODx 
  
  // Loop until XT1,XT2 & DCO stabilizes
  do
  {
    UCSCTL7 &= ~(XT2OFFG + XT1LFOFFG + DCOFFG);
                                            // Clear XT2,DCO fault flags
    SFRIFG1 &= ~OFIFG;                      // Clear fault flags
    TI_LMT01_TEST_LED_PxOUT ^= TI_LMT01_TEST_LED_PIN;                      // Toggle TEST_LED
  }while (SFRIFG1&OFIFG);                   // Test oscillator fault flag
  
  __bis_SR_register(SCG0);                                           // Disable the FLL control loop
  UCSCTL1 = DCORSEL_5;                                               // Select DCO range 16MHz operation
  UCSCTL2 |= 124;                                                    // Set DCO Multiplier for 8MHz
                                                                     // (N + 1) * FLLRef = Fdco
                                                                     // (249 + 1) * 32768 = 8MHz
                                                                     // (124 + 1) * 32768 = 4MHz
  __bic_SR_register(SCG0);                                           // Enable the FLL control loop

  // Worst-case settling time for the DCO when the DCO range bits have been
  // changed is n x 32 x 32 x f_MCLK / f_FLL_reference. See UCS chapter in 5xx
  // UG for optimization.
  // 32 x 32 x 8 MHz / 32,768 Hz = 250000 = MCLK cycles for DCO to settle
  __delay_cycles(250000);                                            // vishy:continue to keep 8MHz delay
  
                                            
  UCSCTL4 = SELA__DCOCLKDIV + SELS__XT2CLK + SELM__XT2CLK;                    // SMCLK=MCLK=XT2 = 4MHz

  // After changing UCSCTL4, loop again until XT1,XT2 & DCO stabilizes (Errat workaround?)
  do
  {
    UCSCTL7 &= ~(XT2OFFG + XT1LFOFFG + DCOFFG);
                                            // Clear XT2,DCO fault flags
    SFRIFG1 &= ~OFIFG;                      // Clear fault flags
    TI_LMT01_TEST_LED_PxOUT ^= TI_LMT01_TEST_LED_PIN;                      // Toggle TEST_LED
  }while (SFRIFG1&OFIFG);                   // Test oscillator fault flag   
  
  TI_LMT01_TEST_LED_PxOUT |= TI_LMT01_TEST_LED_PIN;                        // Turn on TEST_LED   
           
}
//******************************************************************************

/*  
 * ======== UNMI_ISR ========
 */
#pragma vector = UNMI_VECTOR
__interrupt VOID UNMI_ISR (VOID)
{
    switch (__even_in_range(SYSUNIV, SYSUNIV_BUSIFG ))
    {
        case SYSUNIV_NONE:
            __no_operation();
            break;
        case SYSUNIV_NMIIFG:
            __no_operation();
            break;
        case SYSUNIV_OFIFG:
            UCSCTL7 &= ~(DCOFFG + XT1LFOFFG + XT2OFFG); //Clear OSC flaut Flags fault flags
            SFRIFG1 &= ~OFIFG;                          //Clear OFIFG fault flag
            break;
        case SYSUNIV_ACCVIFG:
            __no_operation();
            break;
        case SYSUNIV_BUSIFG:
            SYSBERRIV = 0;                                      //clear bus error flag
            USB_disable();                                      //Disable
    }
}
//******************************************************************************
//EOF