/* --COPYRIGHT--,BSD
* Copyright (c) 2014, Texas Instruments Incorporated
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
* * Redistributions of source code must retain the above copyright
* notice, this list of conditions and the following disclaimer.
*
* * Redistributions in binary form must reproduce the above copyright
* notice, this list of conditions and the following disclaimer in the
* documentation and/or other materials provided with the distribution.
*
* * Neither the name of Texas Instruments Incorporated nor the names of
* its contributors may be used to endorse or promote products derived
* from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
* OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* --/COPYRIGHT--*/
//------------------------------------------------------------------------------

#define Firmware_VersionA 0
#define Firmware_VersionB 1                            // Major Rev: A + B
#define Firmware_VersionC 0
#define Firmware_VersionD 21                            // Minor Rev: C + D

uint8_t handleHostCommand(uint8_t *pBuf, uint16_t pSize);

//******************************************************************************
//Command and Packet definition
// 

#define Command_LoopPacket 0                // Loop back entire packet
											    // PacketBuffer[0],[1] Command
											    // 

#define Command_ReInit 1                    // Controls the firmware settings
											    // PacketBuffer[0],[1] Command
                                                                                            // PacketBuffer[1] Look for external trigger or not in continuous sampling 
                                                                                            // PacketBuffer[2]     													                                            	                                                	
                                                	                                                                                                    
#define Command_SPI_Byte_Write 2            // Writes data to SPI port
											    // PacketBuffer[0],[1] Command
											    // PacketBuffer[1],[2] SPI Address Byte											    
                                                                                            // PacketBuffer[3],[4] SPI Write Data Byte
  

#define Command_SPI_Byte_Read 3             // Read byte data from SPI port 
											    // PacketBuffer[0],[1] Command
											    // PacketBuffer[1],[2] SPI Address Byte
                                                                                            // PacketBuffer[8] Response: SPI Data Byte Read from device

#define Command_Read_Samples_One_Shot 5                 //Start Read Samples one shot command
											    // PacketBuffer[0],[1] Command
											    // 
                                                                             
                                                                                            // PacketBuffer[8] Response: Sample0 low byte 
                                                                                            // PacketBuffer[9] Response: Sample0 high byte
                                                                                            // PacketBuffer[10] Response: Sample1 low byte
                                                                                            // PacketBuffer[11] Response: Sample1 high byte
                                                                                            //  and so onup to Number of Samples one shot (max 8 samples)
                                                                                            //  


#define Command_Start_Graph 6   // Start tof graph
											                                                // PacketBuffer[0],[1] Command
											                                                //

                                                                                            // PacketBuffer[8] Response:


#define Command_End_Graph 7   // stop tof graph
											                                                // PacketBuffer[0],[1] Command
											                                                //
                                                                                            // PacketBuffer[8] Response:

#define Command_Firmware_Version_Read 9     // Read firmware version 
											    // PacketBuffer[0],[1] Command
                                                                                            // PacketBuffer[8] Response: Firmware VersionA 
                                                                                            // PacketBuffer[9] Response: Firmware VersionB
                                                                                            // PacketBuffer[10] Response: Firmware VersionC 
                                                                                            // PacketBuffer[11] Response: Firmware VersionD

#define Command_LED_Toggle 0x0A                // Toggle LED
											    // PacketBuffer[0],[1] Command
											    //

#define Command_Read_Num_Samp_Oneshot 0x0B      // Read Number of Samples in One Shot  
											    // PacketBuffer[0],[1] Command                            
                                                                                            // PacketBuffer[8] Response: Number of Samples   
 

#define Command_Write_Num_Samp_Oneshot 0x0C      // Write Number of Samples in One Shot 
											    // PacketBuffer[0],[1] Command
											    // PacketBuffer[2],[3] Number of Samples
#define Command_Read_Comparator_Low_Threshold 0x0D   // Read Comparator_Low_Threshold 
											    // PacketBuffer[0],[1] Command                            
                                                                                            // PacketBuffer[8] Response: Comparator_Low_Threshold

#define Command_Write_Comparator_Low_Threshold 0x0E   // Write Comparator_Low_Threshold 
											    // PacketBuffer[0],[1] Command                            
                                                                                            // PacketBuffer[2],[3]: Comparator_Low_Threshold

#define Command_Read_Comparator_High_Threshold 0x0F   // Read Comparator_High_Threshold 
											    // PacketBuffer[0],[1] Command                            
                                                                                            // PacketBuffer[8] Response: Comparator_High_Threshold

#define Command_Write_Comparator_High_Threshold 0x10   // Write Comparator_High_Threshold 
											    // PacketBuffer[0],[1] Command                            
                                                                                            // PacketBuffer[2],[3]: Comparator_High_Threshold

#define Command_Read_Comparator_Reference_Level 0x11         // Read Comparator_Reference 
											    // PacketBuffer[0],[1] Command                            
                                                                                            // PacketBuffer[8] Response: Comparator_Reference

#define Command_Write_Comparator_Reference_Level 0x12       // Write Comparator_Reference 
											    // PacketBuffer[0],[1] Command                            
                                                                                            // PacketBuffer[2],[3]: Comparator_Reference
#define Command_Read_Comparator_Filter_Delay 0x13         // Read Comparator_Filter_Delay 
											    // PacketBuffer[0],[1] Command                            
                                                                                            // PacketBuffer[8] Response: Comparator_Filter_Delay

#define Command_Write_Comparator_Filter_Delay 0x14       // Write Comparator_Filter_Delay 
											    // PacketBuffer[0],[1] Command                            
                                                                                            // PacketBuffer[2],[3]: Comparator_Filter_Delay

#define Command_Read_Comparator_Minus_Input_Select 0x15         // Read Comparator_Minus_Input_Select 
											    // PacketBuffer[0],[1] Command                            
                                                                                            // PacketBuffer[8] Response: Comparator_Minus_Input_Select

#define Command_Write_Comparator_Minus_Input_Select 0x16       // Write Comparator_Minus_Input_Select 
											    // PacketBuffer[0],[1] Command                            
                                                                                            // PacketBuffer[2],[3]: Comparator_Minus_Input_Select

#define Command_Set_Timer_Trigger_Freq 0x18
                                                                               // PacketBuffer[0],[1] Command
                                                                               // PacketBuffer[2],[3] trigger freq low byte
                                                                               // PacketBuffer[4],[5] trigger freq high byte

#define Command_Read_Timer_Trigger_Freq 0x1B
                                                                               // PacketBuffer[0],[1] Command
                                                                               // PacketBuffer[8],[9] Timer Trigger Freq byte

#define Command_Enter_MSP430_BSL 0x1D
                                                                               // PacketBuffer[0],[1] Command
                                                                               // 

#define Command_MSP430SPI_Config_Read 0x1E      // Read MSP430 SPI config info 
											    // PacketBuffer[0],[1] Command                            
                                                                                            // PacketBuffer[8] Response: SPI Clock Config   
                                                                                            // PacketBuffer[9] Response: SPI clock divider (low byte)
                                                                                            // PacketBuffer[10] Response: SPI clock divider (high byte)

#define Command_MSP430SPI_Config_Write 0x1F      // Write MSP430 SPI config info 
											    // PacketBuffer[0],[1] Command
											    // PacketBuffer[2],[3] SPI Clock Config
                                                                                            // PacketBuffer[4],[5] SPI Clock divider low byte
                                                                                            // PacketBuffer[6],[7] SPI Clock divider high byte

// codes 0x20 to 0x25 used for gpio/resistor/cap control of Validation Board
#define Command_Set_LMT01_Measure_Mode 0x26     // Set LMT01 Measure Mode
                                                                               // PacketBuffer[0],[1] Command
                                                                               // PacketBuffer[2],[3] desired mode
                                                                               // PacketBuffer[8] Response:

#define Command_Read_LMT01_Measure_Mode 0x27         // Read LMT01 Measure Mode 
											    // PacketBuffer[0],[1] Command                            
                                                                                            // PacketBuffer[8] Response: LMT01 Measure Mode

#define Command_Read_Comparator_Power_Mode 0x28         // Read Comparator_Power_Mode 
											    // PacketBuffer[0],[1] Command                            
                                                                                            // PacketBuffer[8] Response: Comparator_Power_Mode

#define Command_Write_Comparator_Power_Mode 0x29       // Write Comparator_Power_Mode 
											    // PacketBuffer[0],[1] Command                            
                                                                                            // PacketBuffer[2],[3]: Comparator_Power_Mode

#define Command_Read_Comparator_Control_Register 0x2A         // Read Comparator_Control_Register 
											    // PacketBuffer[0],[1] Command 
                                                                                            // PacketBuffer[2],[3] Register to read
                                                                                            // PacketBuffer[8]-[9] Response: Comparator_Control_Register

#define Command_Write_Comparator_Control_Register 0x2B       // Write Comparator_Control_Register 
											    // PacketBuffer[0],[1] Command                            
                                                                                            // PacketBuffer[2],[3]: Register to write
                                                                                            // PacketBuffer[4-7] Register value to write
