/* --COPYRIGHT--,BSD
* Copyright (c) 2014, Texas Instruments Incorporated
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
* * Redistributions of source code must retain the above copyright
* notice, this list of conditions and the following disclaimer.
*
* * Redistributions in binary form must reproduce the above copyright
* notice, this list of conditions and the following disclaimer in the
* documentation and/or other materials provided with the distribution.
*
* * Neither the name of Texas Instruments Incorporated nor the names of
* its contributors may be used to endorse or promote products derived
* from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
* OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* --/COPYRIGHT--*/
//******************************************************************************
#include <stdint.h>
#include "comparator.h"
#include "timer0.h"
#include "timer2.h"
#include "TI_MSP430.h"
#include "TI_MSP430_hardware_board.h"
//******************************************************************************
//******************************************************************************
// Later disable comparatorB interrupt & CB output on P1.6
// Internal VREF is output on P2.5, Input is P2.3
//******************************************************************************
void compB_init(void)
{
  
  // Enable cmpout special function on port pin
  TI_LMT01_CMPB_OUT_PxSEL |= TI_LMT01_CMPB_OUT_PIN;
  TI_LMT01_CMPB_OUT_PxDIR |= TI_LMT01_CMPB_OUT_PIN;
  
  // Setup ComparatorB: Enable V+, input channel CB7 
  CBCTL0 |= CBIPEN + CBIPSEL_7; 
  
  // Input Buffer Disable @P6.7/CB7
  CBCTL3 |= TI_LMT01_CMPB_INP_PIN;  
  
  // Select VeREF+ on P5.0: make pin output w/ special function
  // TI_LMT01_GPIO_INTVREF_PxSEL |= TI_LMT01_GPIO_INTVREF_PIN;
  // TI_LMT01_GPIO_INTVREF_PxDIR |= TI_LMT01_GPIO_INTVREF_PIN;

  // Reference is enabled automatically on selecting shared reference
  while(REFCTL0 & REFGENBUSY);                                                 // If ref generator busy, WAIT                                          
  REFCTL0 |= REFON;                                                            // Internal Reference ON
  REFCTL0 |= REFVSEL_1;                                               // Bring out reference, Select 2.0V
  __delay_cycles(3000);                                                        // Delay for shared ref to stabilize (with 8Mhz/13Mhz/26MHz clock)
  

  // VREF on V-, R-ladder on; bandgap ref voltage (1.5V)
  CBCTL2 |= CBRSEL+ CBRS_2+CBREFL_1; 
  // Tap of resistor string when CBOUT=0: 10/32 & CBOUT=1: 9/32
  CBCTL2 |= CBREF0_9 + CBREF1_8;                
 
  // Output selects between VREF0 or VREF1
  CBCTL1 &= ~CBMRVS; 
  CBCTL1 |= CBPWRMD_0;                      // high speed mode  
  
  // Enable ComparatorB input pin 
  CBCTL1 |= CBON;                                                              // Turn On ComparatorB
  __delay_cycles(100);    

  
}
//******************************************************************************
