/* --COPYRIGHT--,BSD
* Copyright (c) 2014, Texas Instruments Incorporated
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
* * Redistributions of source code must retain the above copyright
* notice, this list of conditions and the following disclaimer.
*
* * Redistributions in binary form must reproduce the above copyright
* notice, this list of conditions and the following disclaimer in the
* documentation and/or other materials provided with the distribution.
*
* * Neither the name of Texas Instruments Incorporated nor the names of
* its contributors may be used to endorse or promote products derived
* from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
* OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* --/COPYRIGHT--*/
//******************************************************************************
#include <intrinsics.h>
#include <stdint.h>
#include "USB_API/USB_Common/types.h"               //Basic Type declarations
#include "TI_MSP430.h"
#include "TI_MSP430_hardware_board.h"
#include "host_interface.h"
#include "timer0.h"
#include "timer2.h"
#include "lmt01_2timer_transistor_modes.h"
#include "lmt01_1gpio_transistor_modes.h"

extern uint8_t single_shot_state;
volatile uint16_t isr_pulse_count=0;
volatile uint8_t isr_pulse_detect=0;
extern uint8_t lmt01_measure_mode;

//******************************************************************************
void lmt01_oneshot_1gpio_sync_transistor_mode(uint8_t *pBuf)
{ 
  static uint8_t blink2=0;
  uint16_t i;
  
  single_shot_state = 1;

  // start afresh: remove & apply power
  lmt01_remove_power();   
  // apply power
  lmt01_apply_power();

  // clear & enable port interrupt
  TI_LMT01_GPIO_PULSE_COUNT_TRAN_PxIFG &= ~TI_LMT01_GPIO_PULSE_COUNT_TRAN_PIN;
  TI_LMT01_GPIO_PULSE_COUNT_TRAN_PxIE |= TI_LMT01_GPIO_PULSE_COUNT_TRAN_PIN;
  
  // reset the pulse counter
  isr_pulse_count = 0;
  // wait 105ms
  for(i=0; i<=104; i++)
     __delay_cycles(4000);  // 1ms at 4MHz
  
  // disable port interrupt
  TI_LMT01_GPIO_PULSE_COUNT_TRAN_PxIE &= ~TI_LMT01_GPIO_PULSE_COUNT_TRAN_PIN;
  // save the result into output buffer
  pBuf[8] = isr_pulse_count;
  pBuf[9] = isr_pulse_count>>8;
  
  // Blink each LED 2 times for 1 GPIO Sync Mode
  if (blink2 < 4)
  {
     TI_LMT01_XFER_LED_PxOUT ^= TI_LMT01_XFER_LED_PIN;                // Set XFER_LED
     blink2++;
  } else if (blink2 < 8)
  {
     TI_LMT01_TEST_LED_PxOUT ^= TI_LMT01_TEST_LED_PIN;                // Set TEST_LED
     blink2++;
  } else
     blink2 = 0;

  single_shot_state = 0;
  // remove power
  lmt01_remove_power();        
    
}
//******************************************************************************
void lmt01_oneshot_1gpio_async_transistor_mode(uint8_t *pBuf)
{
  static uint8_t blink4=0;
  
  single_shot_state = 1;     
  // clear and enable port interrupt
  TI_LMT01_GPIO_PULSE_COUNT_TRAN_PxIFG &= ~TI_LMT01_GPIO_PULSE_COUNT_TRAN_PIN;
  TI_LMT01_GPIO_PULSE_COUNT_TRAN_PxIE |= TI_LMT01_GPIO_PULSE_COUNT_TRAN_PIN;          
  
  // sync twice as first time sync might happen in the middle of a 
  // pulse train - so discard
  sync_to_pulse_train();
  sync_to_pulse_train();
  
  // disable port interrupt
  TI_LMT01_GPIO_PULSE_COUNT_TRAN_PxIE &= ~TI_LMT01_GPIO_PULSE_COUNT_TRAN_PIN;
  // save the result into output buffer
  pBuf[8] = isr_pulse_count;
  pBuf[9] = isr_pulse_count>>8; 
  
  // Blink each LED 4 times for 1 GPIO ASync Mode
  if (blink4 < 8)
  {
     TI_LMT01_XFER_LED_PxOUT ^= TI_LMT01_XFER_LED_PIN;                // Set XFER_LED
     blink4++;
  } else if (blink4 < 16)
  {
     TI_LMT01_TEST_LED_PxOUT ^= TI_LMT01_TEST_LED_PIN;                // Set TEST_LED
     blink4++;
  } else
     blink4 = 0;                         
  
  single_shot_state = 0;  
}
//******************************************************************************
void sync_to_pulse_train(void)
{
  uint32_t tout;
  
  // reset the pulse counter
  isr_pulse_count = 0;  
  tout = 0;
  // look for 90kHz pulse train
  while (1)
  {
     tout++;
     isr_pulse_detect = 0;
     // wait 12us: 48 cycles at 4MHz clock
     __delay_cycles(48);
     if (isr_pulse_detect)
        break;
    // tout defautl is 200000, wait atleast 2sec and timeout
    if (tout >= TIMEOUT_VALUE)
      break;     
  }
  
  // look for end of pulse train + ~100us
  // Note: We look for 90kHz pulse train first and then end of pulse train. 
  // Q: Why not look for end of pulse train first? If we look for end of pulse 
  // train first, then sometimes we may detect a 100us period just before start 
  // of a new pulse train. In that case, due to timing dependency, we may miss
  // a pulse or two before everything is reconfigured to start counting  
  // Looking for a 100us pause after detecting 90kHz pulse train always ensures 
  // we are at the beginning of the LMT01 conversion period and theres plenty 
  // of setup time (50ms+) before the next pulse train begins and no pulses 
  // are missed.
  tout = 0;
  while (1)
  {
     tout++;    
     isr_pulse_detect = 0;
     // wait 100us: 400 cycles at 4MHz clock
     __delay_cycles(400);
     if (!isr_pulse_detect)
        break;
    // tout defautl is 200000, wait atleast 2sec and timeout
    if (tout >= TIMEOUT_VALUE)
      break;     
  }  
}
//******************************************************************************
void init_1gpio_mode_input(void)
{
   switch (lmt01_measure_mode) 
   {
     case SINGLE_GPIO_SYNC_TRANSISTOR_MODE:
     case SINGLE_GPIO_ASYNC_TRANSISTOR_MODE:
     case SINGLE_TIMER_SYNC_TRANSISTOR_MODE:
     case SINGLE_TIMER_ASYNC_TRANSISTOR_MODE:
     case TWO_TIMER_SYNC_TRANSISTOR_MODE:
     case TWO_TIMER_ASYNC_TRANSISTOR_MODE:
       {
         // Init 1 Pulse Count GPIO: input, disable special function                            
         TI_LMT01_GPIO_PULSE_COUNT_TRAN_PxDIR &= ~TI_LMT01_GPIO_PULSE_COUNT_TRAN_PIN; 
         TI_LMT01_GPIO_PULSE_COUNT_TRAN_PxSEL &= ~TI_LMT01_GPIO_PULSE_COUNT_TRAN_PIN;
         TI_LMT01_GPIO_PULSE_COUNT_TRAN_PxIES |= TI_LMT01_GPIO_PULSE_COUNT_TRAN_PIN;           // Interrupt Edge Select
         TI_LMT01_GPIO_PULSE_COUNT_TRAN_PxIFG &= ~TI_LMT01_GPIO_PULSE_COUNT_TRAN_PIN;  
         break;
       }
     case SINGLE_GPIO_SYNC_COMPARATOR_MODE:
     case SINGLE_GPIO_ASYNC_COMPARATOR_MODE:
     case SINGLE_TIMER_SYNC_COMPARATOR_MODE:
     case SINGLE_TIMER_ASYNC_COMPARATOR_MODE:       
     case TWO_TIMER_SYNC_COMPARATOR_MODE:  
     case TWO_TIMER_ASYNC_COMPARATOR_MODE:
       {
         // Init 1 Pulse Count GPIO: input, disable special function                            
         TI_LMT01_GPIO_PULSE_COUNT_COMP_PxDIR &= ~TI_LMT01_GPIO_PULSE_COUNT_COMP_PIN; 
         TI_LMT01_GPIO_PULSE_COUNT_COMP_PxSEL &= ~TI_LMT01_GPIO_PULSE_COUNT_COMP_PIN;
         TI_LMT01_GPIO_PULSE_COUNT_COMP_PxIES |= TI_LMT01_GPIO_PULSE_COUNT_COMP_PIN;      
         TI_LMT01_GPIO_PULSE_COUNT_COMP_PxIFG &= ~TI_LMT01_GPIO_PULSE_COUNT_COMP_PIN;
         break;
       }
   default: break;
   }
}
//******************************************************************************
// 1gpio pulse count interrupt service routine
#pragma vector=TI_LMT01_GPIO_PULSE_COUNT_TRAN_VECTOR
__interrupt void TI_LMT01_PC_PORTx(void)
{

  TI_LMT01_GPIO_PULSE_COUNT_TRAN_PxIFG &= ~TI_LMT01_GPIO_PULSE_COUNT_TRAN_PIN; 
  isr_pulse_count++;
  isr_pulse_detect = 1;
}

//******************************************************************************