/* --COPYRIGHT--,BSD
* Copyright (c) 2014, Texas Instruments Incorporated
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
* * Redistributions of source code must retain the above copyright
* notice, this list of conditions and the following disclaimer.
*
* * Redistributions in binary form must reproduce the above copyright
* notice, this list of conditions and the following disclaimer in the
* documentation and/or other materials provided with the distribution.
*
* * Neither the name of Texas Instruments Incorporated nor the names of
* its contributors may be used to endorse or promote products derived
* from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
* OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* --/COPYRIGHT--*/
//******************************************************************************
#include <intrinsics.h>
#include <stdint.h>
#include "USB_API/USB_Common/types.h"               //Basic Type declarations
#include "TI_MSP430.h"
#include "TI_MSP430_hardware_board.h"
#include "host_interface.h"
#include "timer0.h"
#include "timer2.h"
#include "lmt01_1gpio_transistor_modes.h"
#include "lmt01_2timer_transistor_modes.h"

//******************************************************************************

// at 4MHz system clock, 1ms = 4000, powerdown_wait = 50ms
#define POWERDOWN_WAIT 200000

//******************************************************************************

extern uint8_t single_shot_state;
extern uint8_t xy_buf_length;
extern uint8_t *ubuf;
extern volatile uint8_t LMT01_XYBufferReady;
extern uint8_t lmt01_measure_mode;

extern volatile uint16_t isr_pulse_count;
extern volatile uint8_t isr_pulse_detect;

//******************************************************************************
void lmt01_oneshot_2timer_sync_transistor_mode(uint8_t *pBuf)
{
  uint32_t tout = 0;
  
  single_shot_state = 1;
  LMT01_XYBufferReady = 0; 
  
  // restart Pulse Count Timer and Stop Watch Timer
  lmt01_restart_timers();

  
  while (LMT01_XYBufferReady != 1)
  {
    tout++;
    // 10us delay below at 24MHz master clock
    __delay_cycles(240);
    // tout defautl is 200000, wait atleast 2sec and timeout
    if (tout >= TIMEOUT_VALUE)
      break;        
  }
  if (LMT01_XYBufferReady)
  {
    uint8_t i;
    LMT01_XYBufferReady = 0;
    for(i = 0; i < xy_buf_length; i++)
      pBuf[i+8] = ubuf[i];
  } else
  {
    uint8_t i;
    for(i = 0; i < xy_buf_length; i++)
      pBuf[i+8] = 0;        
  }
  single_shot_state = 0;       
  
  // stop Pulse Count Timer and Stop Watch Timer
  lmt01_stop_timers();   
}
//******************************************************************************
void lmt01_oneshot_2timer_async_transistor_mode(uint8_t *pBuf)
{
  uint32_t tout = 0;
  
  single_shot_state = 1;
  LMT01_XYBufferReady = 0;   
  
  // restart Pulse Count Timer and Stop Watch Timer
  lmt01_restart_timers();
  
  while (LMT01_XYBufferReady != 1)
  {
    tout++;
    // 10us delay below at 24MHz master clock
    __delay_cycles(240);
    // tout defautl is 200000, wait atleast 2sec and timeout
    if (tout >= TIMEOUT_VALUE)
      break;        
  }
  if (LMT01_XYBufferReady)
  {
    uint8_t i;
    LMT01_XYBufferReady = 0;
    for(i = 0; i < xy_buf_length; i++)
      pBuf[i+8] = ubuf[i];
  } else
  {
    uint8_t i;
    for(i = 0; i < xy_buf_length; i++)
      pBuf[i+8] = 0;        
  }
  single_shot_state = 0;        
  
  // stop Pulse Count Timer and Stop Watch Timer
  lmt01_stop_timers();    
}
//******************************************************************************
void lmt01_restart_timers(void)
{
  if ((lmt01_measure_mode == TWO_TIMER_SYNC_TRANSISTOR_MODE) ||
      (lmt01_measure_mode == TWO_TIMER_SYNC_COMPARATOR_MODE))
  {
     // start afresh: remove & apply power
     lmt01_remove_power();   
     // apply power
     lmt01_apply_power();
     
  } else if(lmt01_measure_mode == TWO_TIMER_ASYNC_TRANSISTOR_MODE)
  {
    // Init 1GPIO for pulse count 
    init_1gpio_mode_input();
    // enable port interrupt  
    TI_LMT01_GPIO_PULSE_COUNT_TRAN_PxIE |= TI_LMT01_GPIO_PULSE_COUNT_TRAN_PIN;   
    
    sync_to_pulse_train(); 
    
    // disable port interrupt
    TI_LMT01_GPIO_PULSE_COUNT_TRAN_PxIE &= ~TI_LMT01_GPIO_PULSE_COUNT_TRAN_PIN;
    
    // Init 2timer input for pulse count
    init_2timer_mode_input();
    
  } else if (lmt01_measure_mode == TWO_TIMER_ASYNC_COMPARATOR_MODE) 
  {
    // Init 1GPIO for pulse count 
    init_1gpio_mode_input();
    // enable port interrupt 
    TI_LMT01_GPIO_PULSE_COUNT_COMP_PxIE |= TI_LMT01_GPIO_PULSE_COUNT_COMP_PIN;         
    
    sync_to_pulse_train();

    // disable port interrupt
    TI_LMT01_GPIO_PULSE_COUNT_COMP_PxIE &= ~TI_LMT01_GPIO_PULSE_COUNT_COMP_PIN; 

    // Init 2timer input for pulse count
    init_2timer_mode_input();        
  }

  if ((lmt01_measure_mode == TWO_TIMER_SYNC_TRANSISTOR_MODE) ||
      (lmt01_measure_mode == TWO_TIMER_ASYNC_TRANSISTOR_MODE))
  {
    // clear & restart (previously stoppped) pulse count timer
    TA0CTL |= TACLR;
    TA0CTL |= MC_2;            
    // TA2CCR0 interrupt disabled        
    TA2CCTL0 &= ~CCIE;  
    TA2CCTL0 &= ~CCIFG;
    // enable TA2 Capture1 interrupt
    TA2CCTL1 &= ~CCIFG;
    TA2CCTL1 |= CAP + CCIE;
    // stop & restart 50ms stop watch timer
    TA2CTL |= TACLR + MC_1;     
  } else if ((lmt01_measure_mode == TWO_TIMER_SYNC_COMPARATOR_MODE) ||
             (lmt01_measure_mode == TWO_TIMER_ASYNC_COMPARATOR_MODE))
  {
    // clear & restart (previously stoppped) pulse count timer
    TA2CTL |= TACLR;
    TA2CTL |= MC_2;            
    // TA0CCR0 interrupt disabled        
    TA0CCTL0 &= ~CCIE;  
    TA0CCTL0 &= ~CCIFG;
    // enable TA0 Capture1 interrupt
    TA0CCTL1 &= ~CCIFG;
    TA0CCTL1 |= CAP + CCIE;
    // stop & restart 50ms stop watch timer
    TA0CTL |= TACLR + MC_1;     
  }
}
//******************************************************************************
void lmt01_stop_timers(void)
{
  if ((lmt01_measure_mode == TWO_TIMER_SYNC_TRANSISTOR_MODE) ||
      (lmt01_measure_mode == TWO_TIMER_ASYNC_TRANSISTOR_MODE))
  {  
    // stop pulse count timer
    TA0CTL &= ~MC_2;      
    // TA2CCR0 interrupt disabled        
    TA2CCTL0 &= ~CCIE;      
    // disable capture interrupt 
    TA2CCTL1 &= ~CCIE;
    TA2CCTL1 &= ~CAP;
    // stop 50ms stop watch timer
    TA2CTL &= ~MC_1;    
  } else if ((lmt01_measure_mode == TWO_TIMER_SYNC_COMPARATOR_MODE) ||
             (lmt01_measure_mode == TWO_TIMER_ASYNC_COMPARATOR_MODE))
  {      
    // stop pulse count timer
    TA2CTL &= ~MC_2;      
    // TA0CCR0 interrupt disabled        
    TA0CCTL0 &= ~CCIE;      
    // disable capture interrupt 
    TA0CCTL1 &= ~CCIE;
    TA0CCTL1 &= ~CAP;
    // stop 50ms stop watch timer
    TA0CTL &= ~MC_1;    
  }
  // remove power
  if ((lmt01_measure_mode == TWO_TIMER_SYNC_TRANSISTOR_MODE) ||
      (lmt01_measure_mode == TWO_TIMER_SYNC_COMPARATOR_MODE))  
    lmt01_remove_power();   
}
//****************************************************************************** 
void lmt01_apply_power(void)
{
  // apply power
  TI_LMT01_GPIO_POWER_PxOUT |= TI_LMT01_GPIO_POWER_PIN;
  // atleast 10us * 4 blanking for power to stabilize
  // At 4MHz master clock, 4 delay cycles = 1us
  // wait 1ms
  __delay_cycles(4000);  
}
//******************************************************************************
void lmt01_remove_power(void)
{
  // remove power
  TI_LMT01_GPIO_POWER_PxOUT &= ~TI_LMT01_GPIO_POWER_PIN;
  // Powerdown Wait: power cannnot be applied before this wait 
  __delay_cycles(POWERDOWN_WAIT);
}
//******************************************************************************
void init_2timer_mode_input(void)
{
  
   if ((lmt01_measure_mode == TWO_TIMER_SYNC_TRANSISTOR_MODE) ||
       (lmt01_measure_mode == TWO_TIMER_ASYNC_TRANSISTOR_MODE))
   {        
      // Init Time Measurement GPIO: input & select special function (Transistor Mode)                            
      TI_LMT01_GPIO_PULSE_COUNT_TRAN_PxDIR &= ~TI_LMT01_GPIO_PULSE_COUNT_TRAN_PIN; 
      TI_LMT01_GPIO_PULSE_COUNT_TRAN_PxSEL |= TI_LMT01_GPIO_PULSE_COUNT_TRAN_PIN;  

      // Init CAP_INP GPIO: input & select special function: timer capture input (Transistor Mode)
      TI_LMT01_GPIO_CAP_INP_TRAN_PxDIR &= ~TI_LMT01_GPIO_CAP_INP_TRAN_PIN;
      TI_LMT01_GPIO_CAP_INP_TRAN_PxSEL |= TI_LMT01_GPIO_CAP_INP_TRAN_PIN;         
      
   } else if ((lmt01_measure_mode == TWO_TIMER_SYNC_COMPARATOR_MODE) ||
             (lmt01_measure_mode == TWO_TIMER_ASYNC_COMPARATOR_MODE))
   {              
      // Init Time Measurement GPIO: input & select special function (Comparator Mode)                            
      TI_LMT01_GPIO_PULSE_COUNT_COMP_PxDIR &= ~TI_LMT01_GPIO_PULSE_COUNT_COMP_PIN; 
      TI_LMT01_GPIO_PULSE_COUNT_COMP_PxSEL |= TI_LMT01_GPIO_PULSE_COUNT_COMP_PIN;
       
      // Init CAP_INP GPIO: input & select special function: timer capture input (Comparator Mode)
      TI_LMT01_GPIO_CAP_INP_COMP_PxDIR &= ~TI_LMT01_GPIO_CAP_INP_COMP_PIN;
      TI_LMT01_GPIO_CAP_INP_COMP_PxSEL |= TI_LMT01_GPIO_CAP_INP_COMP_PIN;                 
      
   }   
}
//******************************************************************************
