/* --COPYRIGHT--,BSD
* Copyright (c) 2014, Texas Instruments Incorporated
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
* * Redistributions of source code must retain the above copyright
* notice, this list of conditions and the following disclaimer.
*
* * Redistributions in binary form must reproduce the above copyright
* notice, this list of conditions and the following disclaimer in the
* documentation and/or other materials provided with the distribution.
*
* * Neither the name of Texas Instruments Incorporated nor the names of
* its contributors may be used to endorse or promote products derived
* from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
* OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* --/COPYRIGHT--*/

#include <stdint.h>
#include "TI_MSP430.h"
#include "TI_MSP430_hardware_board.h"
#include "timer0.h"
#include "timer2.h"
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
extern uint8_t bufferX[], bufferY[];
extern uint8_t *mbuf, *ubuf;
extern uint8_t xy_buf_length;
extern volatile uint8_t LMT01_XYBufferReady;
extern uint8_t single_shot_state;
extern uint8_t cont_graph_state;
extern uint8_t lmt01_measure_mode;

//******************************************************************************
//------------------------------------------------------------------------------
//  void timer2_A0_init(void)
//
//  DESCRIPTION:
//  Initialize timer2_A0 to generate interrupt every 50msec. 
//  Also, pwm output on TA2.1 = P2.4 using TA2CCR1
//------------------------------------------------------------------------------
void timer2_A0_init(void)
{
  if ((lmt01_measure_mode == TWO_TIMER_SYNC_COMPARATOR_MODE) ||
     (lmt01_measure_mode == TWO_TIMER_ASYNC_COMPARATOR_MODE))
  {  
    // Clock source from pin, clear TAR, stop mode, divide by 1, no overflow intrpt
    TA2CTL = TASSEL_0 + TACLR + MC_0 + ID_0; 
    // reset other registers to default value
    TA2CCR0 = 0;
    TA2CCTL1 = 0;
    TA2EX0 = 0;
  } else if ((lmt01_measure_mode == TWO_TIMER_SYNC_TRANSISTOR_MODE) ||
             (lmt01_measure_mode == TWO_TIMER_ASYNC_TRANSISTOR_MODE))
  {                                                            
    // Initialize for 50msec interval
    TA2CCR0 = INTERVAL_55MSEC; 
    // +ve edge; input select: CCIxA=TA2.1=P2.4, capture mode; don't enable intrpt yet
    TA2CCTL1 &= ~CCIE;
    TA2CCTL1 &= ~CCIFG;  
    TA2CCTL1 = CM_1 + CCIS_0 + SCS;
    // Divide clock (selected below) further by 2   
    TA2EX0 |= TAIDEX_1;                                                          	
    // SMCLK (4MHz), stop mode, clear TAR, divide by 8, no overflow interrupt  
    TA2CTL = TASSEL_2 + MC_0 + TACLR + ID_3;                                    
  } else if ((lmt01_measure_mode == SINGLE_TIMER_SYNC_TRANSISTOR_MODE)  ||
             (lmt01_measure_mode == SINGLE_TIMER_ASYNC_TRANSISTOR_MODE) ||
             (lmt01_measure_mode == SINGLE_TIMER_SYNC_COMPARATOR_MODE)  ||
             (lmt01_measure_mode == SINGLE_TIMER_ASYNC_COMPARATOR_MODE))
  {
    // Clock source from pin:TA0CLK=P1.0, clear TAR, stop mode, divide by 1, no overflow intrpt
    TA2CTL = TASSEL_0 + TACLR + MC_0 + ID_0;   
    // reset other registers to default value
    TA2CCR0 = 0;
    TA2CCTL1 = 0;
    TA2EX0 = 0;     
  } else if  ((lmt01_measure_mode == SINGLE_GPIO_SYNC_TRANSISTOR_MODE)   ||
             (lmt01_measure_mode == SINGLE_GPIO_ASYNC_TRANSISTOR_MODE)  ||
             (lmt01_measure_mode == SINGLE_GPIO_SYNC_COMPARATOR_MODE)   ||  
             (lmt01_measure_mode == SINGLE_GPIO_ASYNC_COMPARATOR_MODE))
  {
    // reset registers to default value
    TA2CTL = 0;
    TA2CCR0 = 0;
    TA2CCTL1 = 0;
    TA2EX0 = 0;     
  }
}
//******************************************************************************
// Timer2 A1 interrupt service routine
//------------------------------------------------------------------------------
#pragma vector=TIMER2_A1_VECTOR
__interrupt void TIMER2_A1_ISR(void)
{
  switch(__even_in_range(TA2IV,14))
  {
    case 0: break;                  
    case 2:                                                                    // CCR1 interrupt
            // clear 50ms stop watch timer
            TA2CTL |= TACLR;
            // restart timer2 counter: synchronizes to pulse train beginning
            // disable capture interrupt
            TA2CCTL1 &= ~CCIE;
            // TA2CCR0 interrupt enabled
            TA2CCTL0 &= ~CCIFG;
            TA2CCTL0 |= CCIE;            
//            __bic_SR_register_on_exit(LPM0_bits);                              // Exit LPM0  
             break;
    case 4:  break;                         // CCR2 not used
    case 6:  break;                         // CCR3 not used
    case 8:  break;                         // CCR4 not used
    case 10: break;                         // CCR5 not used
    case 12: break;                         // Reserved not used
    case 14:                                // overflow interrupt
             break;
    default: break;
 }
}
//******************************************************************************
// Timer2 A0 interrupt service routine
// Transistor Mode:50ms stop watch done interrupt
//------------------------------------------------------------------------------
#pragma vector=TIMER2_A0_VECTOR
__interrupt void TIMER2_A0_ISR (void)
{
  static uint8_t plen = 0;
           
  // clear 50ms stop watch timer
  TA2CTL |= TACLR;  
// TA2CCR0 interrupt disabled        
  TA2CCTL0 &= ~CCIE;  
  if ((cont_graph_state == 1)||(single_shot_state ==1))
  {
    TA2CCTL1 &= ~CCIFG;
    TA2CCTL1 |= CCIE;           // enable capture interrupt
    // stop pulse count timer
    TA0CTL &= ~MC_2;
    // save pulse count and clear timer counter 
    if (TA0R != 0)
    {
      mbuf[plen++] = TA0R;
      mbuf[plen++] = TA0R>>8;
    }
    TA0CTL |= TACLR;         
    // restart pulse count timer
    TA0CTL |= MC_2;
    if (plen == xy_buf_length)
    { 
      plen = 0;
      LMT01_XYBufferReady = 1;
      ubuf = mbuf;
      if (mbuf == bufferX)
        mbuf = bufferY;
      else
        mbuf = bufferX;
      if (single_shot_state == 1)
      {
        // stop pulse count timer
        TA0CTL &= ~MC_2;
        // disable capture interrupt                 
        TA2CCTL1 &= ~CCIE;
        single_shot_state = 0;
      }
    }                
  } else
      TA2CCTL1 &= ~CCIE; // disable capture interrupt
// __bic_SR_register_on_exit(LPM0_bits);                                       // Exit LPM0  
  
}
//******************************************************************************
